{-# LANGUAGE LambdaCase #-}
module Operations.List where

import Control.Monad.Buuka
import Control.Monad.Reader
       (asks, liftIO)

import Data.Foldable
       (traverse_)

import Data.Semigroup
       (Max(..))

import Data.Buuka
       (Buuka, BuukaEntry(..), URL(..))
import qualified Data.Buuka as B

import Web.Hashids

import Control.Lens
import Data.Text.Strict.Lens
       (unpacked, utf8)

list :: BuukaM ()
list =
  buukaQ (asks go) >>= traverse_ (liftIO . putStrLn)
  where
    go :: Buuka -> [String]
    go b =
      let ctx = hashidsSimple (B.fingerprint b)
      in format ctx (B.elements b)
    format :: HashidsContext -> [BuukaEntry] -> [String]
    format ctx xs =
      let formatted = zipWith (formatEntry ctx) [1..] xs
          indexWidth = getMax . foldMap (Max . length . fst) $ formatted
      in fmap (\(idx,x) -> idx <> replicate (indexWidth - length idx) ' ' <> ". " <> x) formatted
    formatEntry :: HashidsContext -> Int -> BuukaEntry -> (String, String)
    formatEntry ctx n = \case
      BuukaEntry{title=Just t} -> (encode ctx n ^. utf8 . unpacked, t)
      BuukaEntry{url=URL u} -> (encode ctx n ^. utf8 . unpacked, u)
